@@ -1,5 +1,8 @@
 /*
- * Copyright © 2009 CNRS, INRIA, Université Bordeaux 1
+ * Copyright © 2009 CNRS
+ * Copyright © 2009-2010 INRIA.  All rights reserved.
+ * Copyright © 2009-2011 Université Bordeaux 1
+ * Copyright © 2011 Cisco Systems, Inc.  All rights reserved.
  * See COPYING in top-level directory.
  */
 
@@ -18,10 +21,16 @@
 #include <hwloc/helper.h>
 #include <assert.h>
 
-#if !defined _GNU_SOURCE || !defined _SCHED_H
-#error sched.h must be included with _GNU_SOURCE defined
+#if !defined _GNU_SOURCE || !defined _SCHED_H || (!defined CPU_SETSIZE && !defined sched_priority)
+#error Please make sure to include sched.h before including glibc-sched.h, and define _GNU_SOURCE before any inclusion of sched.h
 #endif
 
+
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+
 #ifdef HWLOC_HAVE_CPU_SET
 
 
@@ -44,16 +53,16 @@ hwloc_cpuset_to_glibc_sched_affinity(hwloc_topology_t topology __hwloc_attribute
 #ifdef CPU_ZERO_S
   unsigned cpu;
   CPU_ZERO_S(schedsetsize, schedset);
-  hwloc_cpuset_foreach_begin(cpu, hwlocset)
+  hwloc_bitmap_foreach_begin(cpu, hwlocset)
     CPU_SET_S(cpu, schedsetsize, schedset);
-  hwloc_cpuset_foreach_end();
+  hwloc_bitmap_foreach_end();
 #else /* !CPU_ZERO_S */
   unsigned cpu;
   CPU_ZERO(schedset);
   assert(schedsetsize == sizeof(cpu_set_t));
-  hwloc_cpuset_foreach_begin(cpu, hwlocset)
+  hwloc_bitmap_foreach_begin(cpu, hwlocset)
     CPU_SET(cpu, schedset);
-  hwloc_cpuset_foreach_end();
+  hwloc_bitmap_foreach_end();
 #endif /* !CPU_ZERO_S */
   return 0;
 }
@@ -69,19 +78,19 @@ static __hwloc_inline int
 hwloc_cpuset_from_glibc_sched_affinity(hwloc_topology_t topology __hwloc_attribute_unused, hwloc_cpuset_t hwlocset,
                                        const cpu_set_t *schedset, size_t schedsetsize)
 {
-  hwloc_cpuset_zero(hwlocset);
 #ifdef CPU_ZERO_S
   int cpu, count;
+#endif
+  hwloc_bitmap_zero(hwlocset);
+#ifdef CPU_ZERO_S
   count = CPU_COUNT_S(schedsetsize, schedset);
   cpu = 0;
   while (count) {
     if (CPU_ISSET_S(cpu, schedsetsize, schedset)) {
-      hwloc_cpuset_set(hwlocset, cpu);
+      hwloc_bitmap_set(hwlocset, cpu);
       count--;
     }
     cpu++;
-    if (cpu > HWLOC_NBMAXCPUS)
-      break;
   }
 #else /* !CPU_ZERO_S */
   /* sched.h does not support dynamic cpu_set_t (introduced in glibc 2.7),
@@ -89,9 +98,9 @@ hwloc_cpuset_from_glibc_sched_affinity(hwloc_topology_t topology __hwloc_attribu
    */
   int cpu;
   assert(schedsetsize == sizeof(cpu_set_t));
-  for(cpu=0; cpu<CPU_SETSIZE && cpu<HWLOC_NBMAXCPUS; cpu++)
+  for(cpu=0; cpu<CPU_SETSIZE; cpu++)
     if (CPU_ISSET(cpu, schedset))
-      hwloc_cpuset_set(hwlocset, cpu);
+      hwloc_bitmap_set(hwlocset, cpu);
 #endif /* !CPU_ZERO_S */
   return 0;
 }
@@ -101,4 +110,10 @@ hwloc_cpuset_from_glibc_sched_affinity(hwloc_topology_t topology __hwloc_attribu
 
 #endif /* CPU_SET */
 
+
+#ifdef __cplusplus
+} /* extern "C" */
+#endif
+
+
 #endif /* HWLOC_GLIBC_SCHED_H */
