@@ -1,6 +1,8 @@
 /*
- * Copyright © 2009 CNRS, INRIA, Université Bordeaux 1
- * Copyright © 2009 Cisco Systems, Inc.  All rights reserved.
+ * Copyright © 2009 CNRS
+ * Copyright © 2009-2011 INRIA.  All rights reserved.
+ * Copyright © 2009-2011 Université Bordeaux 1
+ * Copyright © 2009-2011 Cisco Systems, Inc.  All rights reserved.
  * See COPYING in top-level directory.
  */
 
@@ -9,7 +11,7 @@
 
 /* Apparently, Darwin people do not _want_ to provide binding functions.  */
 
-#include <private/config.h>
+#include <private/autogen/config.h>
 
 #include <sys/types.h>
 #include <sys/sysctl.h>
@@ -30,7 +32,9 @@ hwloc_look_darwin(struct hwloc_topology *topology)
   struct hwloc_obj *obj;
   size_t size;
   int64_t l1cachesize;
+  int64_t cacheways[2];
   int64_t l2cachesize;
+  int64_t cachelinesize;
   int64_t memsize;
 
   if (hwloc_get_sysctlbyname("hw.ncpu", &_nprocs) || _nprocs <= 0)
@@ -60,11 +64,11 @@ hwloc_look_darwin(struct hwloc_topology *topology)
     if (nprocs == npackages * logical_per_package)
       for (i = 0; i < npackages; i++) {
         obj = hwloc_alloc_setup_object(HWLOC_OBJ_SOCKET, i);
-        obj->cpuset = hwloc_cpuset_alloc();
+        obj->cpuset = hwloc_bitmap_alloc();
         for (cpu = i*logical_per_package; cpu < (i+1)*logical_per_package; cpu++)
-          hwloc_cpuset_set(obj->cpuset, cpu);
+          hwloc_bitmap_set(obj->cpuset, cpu);
 
-        hwloc_debug_1arg_cpuset("package %u has cpuset %s\n",
+        hwloc_debug_1arg_bitmap("package %u has cpuset %s\n",
                    i, obj->cpuset);
         hwloc_insert_object_by_cpuset(topology, obj);
       }
@@ -76,13 +80,13 @@ hwloc_look_darwin(struct hwloc_topology *topology)
       if (!(logical_per_package % cores_per_package))
         for (i = 0; i < npackages * cores_per_package; i++) {
           obj = hwloc_alloc_setup_object(HWLOC_OBJ_CORE, i);
-          obj->cpuset = hwloc_cpuset_alloc();
+          obj->cpuset = hwloc_bitmap_alloc();
           for (cpu = i*(logical_per_package/cores_per_package);
                cpu < (i+1)*(logical_per_package/cores_per_package);
                cpu++)
-            hwloc_cpuset_set(obj->cpuset, cpu);
+            hwloc_bitmap_set(obj->cpuset, cpu);
 
-          hwloc_debug_1arg_cpuset("core %u has cpuset %s\n",
+          hwloc_debug_1arg_bitmap("core %u has cpuset %s\n",
                      i, obj->cpuset);
           hwloc_insert_object_by_cpuset(topology, obj);
         }
@@ -95,14 +99,40 @@ hwloc_look_darwin(struct hwloc_topology *topology)
   if (hwloc_get_sysctlbyname("hw.l2cachesize", &l2cachesize))
     l2cachesize = 0;
 
+  if (hwloc_get_sysctlbyname("machdep.cpu.cache.L1_associativity", &cacheways[0]))
+    cacheways[0] = 0;
+  else if (cacheways[0] == 0xff)
+    cacheways[0] = -1;
+
+  if (hwloc_get_sysctlbyname("machdep.cpu.cache.L2_associativity", &cacheways[1]))
+    cacheways[1] = 0;
+  else if (cacheways[1] == 0xff)
+    cacheways[1] = -1;
+
+  if (hwloc_get_sysctlbyname("hw.cachelinesize", &cachelinesize))
+    cachelinesize = 0;
+
   if (hwloc_get_sysctlbyname("hw.memsize", &memsize))
     memsize = 0;
 
   if (!sysctlbyname("hw.cacheconfig", NULL, &size, NULL, 0)) {
     unsigned n = size / sizeof(uint32_t);
-    uint64_t cacheconfig[n];
-    uint32_t cacheconfig32[n];
-    uint64_t cachesize[n];
+    uint64_t *cacheconfig = NULL;
+    uint64_t *cachesize = NULL;
+    uint32_t *cacheconfig32 = NULL;
+
+    cacheconfig = malloc(sizeof(uint64_t) * n);
+    if (NULL == cacheconfig) {
+        goto out;
+    }
+    cachesize = malloc(sizeof(uint64_t) * n);
+    if (NULL == cachesize) {
+        goto out;
+    }
+    cacheconfig32 = malloc(sizeof(uint32_t) * n);
+    if (NULL == cacheconfig32) {
+        goto out;
+    }
 
     if ((!sysctlbyname("hw.cacheconfig", cacheconfig, &size, NULL, 0))) {
       /* Yeech. Darwin seemingly has changed from 32bit to 64bit integers for
@@ -114,8 +144,8 @@ hwloc_look_darwin(struct hwloc_topology *topology)
           cacheconfig[i] = cacheconfig32[i];
       }
 
-      memset(cachesize, 0, sizeof(cachesize));
-      size = sizeof(cachesize);
+      memset(cachesize, 0, sizeof(uint64_t) * n);
+      size = sizeof(uint64_t) * n;
       if (sysctlbyname("hw.cachesize", cachesize, &size, NULL, 0)) {
         if (n > 0)
           cachesize[0] = memsize;
@@ -140,22 +170,27 @@ hwloc_look_darwin(struct hwloc_topology *topology)
         for (j = 0; j < (nprocs / cacheconfig[i]); j++) {
           obj = hwloc_alloc_setup_object(i?HWLOC_OBJ_CACHE:HWLOC_OBJ_NODE, j);
           if (!i) {
-            obj->nodeset = hwloc_cpuset_alloc();
-            hwloc_cpuset_set(obj->nodeset, j);
+            obj->nodeset = hwloc_bitmap_alloc();
+            hwloc_bitmap_set(obj->nodeset, j);
           }
-          obj->cpuset = hwloc_cpuset_alloc();
+          obj->cpuset = hwloc_bitmap_alloc();
           for (cpu = j*cacheconfig[i];
                cpu < ((j+1)*cacheconfig[i]);
                cpu++)
-            hwloc_cpuset_set(obj->cpuset, cpu);
+            hwloc_bitmap_set(obj->cpuset, cpu);
 
           if (i) {
-            hwloc_debug_2args_cpuset("L%ucache %u has cpuset %s\n",
+            hwloc_debug_2args_bitmap("L%ucache %u has cpuset %s\n",
                 i, j, obj->cpuset);
             obj->attr->cache.depth = i;
             obj->attr->cache.size = cachesize[i];
+            obj->attr->cache.linesize = cachelinesize;
+            if (i <= sizeof(cacheways) / sizeof(cacheways[0]))
+              obj->attr->cache.associativity = cacheways[i-1];
+            else
+              obj->attr->cache.associativity = 0;
           } else {
-            hwloc_debug_1arg_cpuset("node %u has cpuset %s\n",
+            hwloc_debug_1arg_bitmap("node %u has cpuset %s\n",
                 j, obj->cpuset);
 	    obj->memory.local_memory = cachesize[i];
 	    obj->memory.page_types_len = 2;
@@ -171,10 +206,23 @@ hwloc_look_darwin(struct hwloc_topology *topology)
         }
       }
     }
+  out:
+    if (NULL != cacheconfig) {
+        free(cacheconfig);
+    }
+    if (NULL != cachesize) {
+        free(cachesize);
+    }
+    if (NULL != cacheconfig32) {
+        free(cacheconfig32);
+    }
   }
 
+
   /* add PU objects */
   hwloc_setup_pu_level(topology, nprocs);
+
+  hwloc_obj_add_info(topology->levels[0][0], "Backend", "Darwin");
 }
 
 void
