@@ -1,5 +1,5 @@
 /*
- * Copyright © 2010 INRIA.  All rights reserved.
+ * Copyright © 2010-2011 INRIA.  All rights reserved.
  * Copyright © 2010-2011 Université Bordeaux 1
  * Copyright © 2010-2011 Cisco Systems, Inc.  All rights reserved.
  * See COPYING in top-level directory.
@@ -28,7 +28,7 @@ struct cacheinfo {
 
   unsigned linesize;
   unsigned linepart;
-  unsigned ways;
+  int ways;
   unsigned sets;
   unsigned size;
 };
@@ -82,10 +82,13 @@ static void fill_amd_cache(struct procinfo *infos, unsigned level, unsigned cpui
     cache->nbthreads_sharing = infos->max_log_proc;
   cache->linesize = cpuid & 0xff;
   cache->linepart = 0;
-  if (level == 1)
+  if (level == 1) {
     cache->ways = (cpuid >> 16) & 0xff;
-  else {
-    static const unsigned ways_tab[] = { 0, 1, 2, 0, 4, 0, 8, 0, 16, 0, 32, 48, 64, 96, 128, 0 };
+    if (cache->ways == 0xff)
+      /* Fully associative */
+      cache->ways = -1;
+  } else {
+    static const unsigned ways_tab[] = { 0, 1, 2, 0, 4, 0, 8, 0, 16, 0, 32, 48, 64, 96, 128, -1 };
     unsigned ways = (cpuid >> 12) & 0xf;
     cache->ways = ways_tab[ways];
   }
@@ -203,7 +206,11 @@ static void look_proc(struct procinfo *infos, unsigned highest_cpuid, unsigned h
 
       cache->linesize = linesize = (ebx & 0xfff) + 1;
       cache->linepart = linepart = ((ebx >> 12) & 0x3ff) + 1;
-      cache->ways = ways = ((ebx >> 22) & 0x3ff) + 1;
+      if (eax & (1 << 9))
+        /* Fully associative */
+        cache->ways = -1;
+      else
+        cache->ways = ways = ((ebx >> 22) & 0x3ff) + 1;
       cache->sets = sets = ecx + 1;
       cache->size = linesize * linepart * ways * sets;
 
@@ -219,7 +226,7 @@ static void look_proc(struct procinfo *infos, unsigned highest_cpuid, unsigned h
   }
 
   if (cpuid_type == intel && highest_cpuid >= 0x0b) {
-    unsigned level, apic_nextshift, apic_number, apic_type, apic_id, apic_shift = 0, id;
+    unsigned level, apic_nextshift, apic_number, apic_type, apic_id = 0, apic_shift = 0, id;
     for (level = 0; ; level++) {
       ecx = level;
       eax = 0x0b;
@@ -270,7 +277,8 @@ static void look_proc(struct procinfo *infos, unsigned highest_cpuid, unsigned h
 static void summarize(hwloc_topology_t topology, struct procinfo *infos, unsigned nbprocs)
 {
   hwloc_bitmap_t complete_cpuset = hwloc_bitmap_alloc();
-  unsigned i, j, l, one, level;
+  unsigned i, j, l, level;
+  int one = -1;
 
   for (i = 0; i < nbprocs; i++)
     if (infos[i].present) {
@@ -278,6 +286,9 @@ static void summarize(hwloc_topology_t topology, struct procinfo *infos, unsigne
       one = i;
     }
 
+  if (one == -1)
+    return;
+
   /* Look for sockets */
   {
     hwloc_bitmap_t sockets_cpuset = hwloc_bitmap_dup(complete_cpuset);
@@ -421,6 +432,7 @@ static void summarize(hwloc_topology_t topology, struct procinfo *infos, unsigne
           cache->attr->cache.depth = level;
           cache->attr->cache.size = infos[i].cache[l].size;
           cache->attr->cache.linesize = infos[i].cache[l].linesize;
+          cache->attr->cache.associativity = infos[i].cache[l].ways;
           cache->cpuset = cache_cpuset;
           hwloc_debug_2args_bitmap("os L%u cache %u has cpuset %s\n",
               level, cacheid, cache_cpuset);
@@ -522,7 +534,7 @@ void hwloc_look_x86(struct hwloc_topology *topology, unsigned nbprocs)
   }
 #endif
 
-  hwloc_add_object_info(topology->levels[0][0], "Backend", "x86");
+  hwloc_obj_add_info(topology->levels[0][0], "Backend", "x86");
 
  free:
   if (NULL != infos) {
