@@ -1,7 +1,7 @@
 /*
  * Copyright © 2009 CNRS
  * Copyright © 2009-2010 INRIA.  All rights reserved.
- * Copyright © 2009-2010 Université Bordeaux 1
+ * Copyright © 2009-2011 Université Bordeaux 1
  * Copyright © 2009-2011 Cisco Systems, Inc.  All rights reserved.
  * See COPYING in top-level directory.
  */
@@ -27,7 +27,11 @@
 #endif /* HAVE_PUTWC */
 
 #ifdef HWLOC_HAVE_LIBTERMCAP
-#include <curses.h>
+#ifdef HWLOC_USE_NCURSES
+#  include <ncurses.h>
+#else
+#  include <curses.h>
+#endif
 #include <term.h>
 #endif /* HWLOC_HAVE_LIBTERMCAP */
 
@@ -54,7 +58,9 @@ output_console_obj (hwloc_obj_t l, FILE *output, int logical, int verbose_mode)
       hwloc_obj_type_snprintf (type, sizeof(type), l, verbose_mode-1);
       fprintf(output, "%s", type);
     }
-    if (l->depth != 0 && idx != (unsigned)-1)
+    if (l->depth != 0 && idx != (unsigned)-1
+        && l->type != HWLOC_OBJ_PCI_DEVICE
+        && (l->type != HWLOC_OBJ_BRIDGE || l->attr->bridge.upstream_type == HWLOC_OBJ_BRIDGE_HOST))
       fprintf(output, "%s%u", indexprefix, idx);
     if (logical && l->os_index != (unsigned) -1 &&
 	(verbose_mode >= 2 || l->type == HWLOC_OBJ_PU || l->type == HWLOC_OBJ_NODE))
@@ -69,7 +75,7 @@ output_console_obj (hwloc_obj_t l, FILE *output, int logical, int verbose_mode)
 	      phys, separator, attr);
     }
     free(attr);
-    if (verbose_mode >= 2 && l->name && l->type != HWLOC_OBJ_MISC)
+    if ((l->type == HWLOC_OBJ_OS_DEVICE || verbose_mode >= 2) && l->name && l->type != HWLOC_OBJ_MISC)
       fprintf(output, " \"%s\"", l->name);
   }
   if (!l->cpuset)
@@ -205,7 +211,7 @@ void output_console(hwloc_topology_t topology, const char *filename, int logical
     hwloc_const_bitmap_t online = hwloc_topology_get_online_cpuset(topology);
     hwloc_const_bitmap_t allowed = hwloc_topology_get_allowed_cpuset(topology);
 
-    if (!hwloc_bitmap_isequal(topo, complete)) {
+    if (complete && !hwloc_bitmap_isequal(topo, complete)) {
       hwloc_bitmap_t unknown = hwloc_bitmap_alloc();
       char *unknownstr;
       hwloc_bitmap_copy(unknown, complete);
@@ -215,7 +221,7 @@ void output_console(hwloc_topology_t topology, const char *filename, int logical
       free(unknownstr);
       hwloc_bitmap_free(unknown);
     }
-    if (!hwloc_bitmap_isequal(online, complete)) {
+    if (complete && !hwloc_bitmap_isequal(online, complete)) {
       hwloc_bitmap_t offline = hwloc_bitmap_alloc();
       char *offlinestr;
       hwloc_bitmap_copy(offline, complete);
@@ -225,7 +231,7 @@ void output_console(hwloc_topology_t topology, const char *filename, int logical
       free(offlinestr);
       hwloc_bitmap_free(offline);
     }
-    if (!hwloc_bitmap_isequal(allowed, online)) {
+    if (complete && !hwloc_bitmap_isequal(allowed, online)) {
       if (!hwloc_bitmap_isincluded(online, allowed)) {
         hwloc_bitmap_t forbidden = hwloc_bitmap_alloc();
         char *forbiddenstr;
@@ -630,8 +636,20 @@ text_text(void *output, int r, int g, int b, int size __hwloc_attribute_unused,
   struct display *disp = output;
   x /= (gridsize/2);
   y /= gridsize;
+
+#ifdef HAVE_PUTWC
+  {
+    size_t len = strlen(text) + 1;
+    wchar_t *wbuf = malloc(len * sizeof(wchar_t)), *wtext;
+    swprintf(wbuf, len, L"%s", text);
+    for (wtext = wbuf ; *wtext; wtext++)
+      put(disp, x++, y, *wtext, r, g, b, -1, -1, -1);
+    free(wbuf);
+  }
+#else
   for ( ; *text; text++)
     put(disp, x++, y, *text, r, g, b, -1, -1, -1);
+#endif
 }
 
 static struct draw_methods text_draw_methods = {
