@@ -56,10 +56,14 @@ FILE *open_file(const char *filename, const char *mode)
   return fopen(filename, mode);
 }
 
-static hwloc_obj_t insert_task(hwloc_topology_t topology, hwloc_const_cpuset_t cpuset, const char * name)
+static hwloc_obj_t insert_task(hwloc_topology_t topology, hwloc_cpuset_t cpuset, const char * name)
 {
   hwloc_obj_t obj;
 
+  hwloc_bitmap_and(cpuset, cpuset, hwloc_topology_get_topology_cpuset(topology));
+  if (hwloc_bitmap_iszero(cpuset))
+    return NULL;
+
   /* try to insert at exact position */
   obj = hwloc_topology_insert_misc_object_by_cpuset(topology, cpuset, name);
   if (!obj) {
@@ -222,9 +226,7 @@ void usage(const char *name, FILE *where)
 		  ", svg"
 #endif /* CAIRO_HAS_SVG_SURFACE */
 #endif /* HWLOC_HAVE_CAIRO */
-#ifdef HWLOC_HAVE_XML
 		  ", xml"
-#endif /* HWLOC_HAVE_XML */
 		  "\n");
   fprintf (where, "\nFormatting options:\n");
   fprintf (where, "  -l --logical          Display hwloc logical object indexes\n");
@@ -250,6 +252,11 @@ void usage(const char *name, FILE *where)
                   "                        impact\n");
   fprintf (where, "  --restrict <cpuset>   Restrict the topology to processors listed in <cpuset>\n");
   fprintf (where, "  --restrict binding    Restrict the topology to the current process binding\n");
+#ifdef HWLOC_HAVE_LIBPCI
+  fprintf (where, "  --no-io               Do not show any I/O device or bridge\n");
+  fprintf (where, "  --no-bridges          Do not any I/O bridge except hostbridges\n");
+  fprintf (where, "  --whole-io            Show all I/O devices and bridges\n");
+#endif
   fprintf (where, "Input options:\n");
   hwloc_utils_input_format_usage(where, 6);
   fprintf (where, "  --thissystem          Assume that the input topology provides the topology\n"
@@ -315,7 +322,7 @@ main (int argc, char *argv[])
   int verbose_mode = LSTOPO_VERBOSE_MODE_DEFAULT;
   hwloc_topology_t topology;
   const char *filename = NULL;
-  unsigned long flags = 0;
+  unsigned long flags = HWLOC_TOPOLOGY_FLAG_IO_DEVICES | HWLOC_TOPOLOGY_FLAG_IO_BRIDGES;
   int merge = 0;
   int ignorecache = 0;
   char * callname;
@@ -383,6 +390,12 @@ main (int argc, char *argv[])
 	ignorecache = 1;
       else if (!strcmp (argv[1], "--whole-system"))
 	flags |= HWLOC_TOPOLOGY_FLAG_WHOLE_SYSTEM;
+      else if (!strcmp (argv[1], "--no-io"))
+	flags &= ~(HWLOC_TOPOLOGY_FLAG_IO_DEVICES | HWLOC_TOPOLOGY_FLAG_IO_BRIDGES);
+      else if (!strcmp (argv[1], "--no-bridges"))
+	flags &= ~(HWLOC_TOPOLOGY_FLAG_IO_BRIDGES);
+      else if (!strcmp (argv[1], "--whole-io"))
+	flags |= HWLOC_TOPOLOGY_FLAG_WHOLE_IO;
       else if (!strcmp (argv[1], "--merge"))
 	merge = 1;
       else if (!strcmp (argv[1], "--thissystem"))
@@ -466,8 +479,11 @@ main (int argc, char *argv[])
   if (merge)
     hwloc_topology_ignore_all_keep_structure(topology);
 
-  if (input)
-    hwloc_utils_enable_input_format(topology, input, input_format, verbose_mode > 1, callname);
+  if (input) {
+    err = hwloc_utils_enable_input_format(topology, input, input_format, verbose_mode > 1, callname);
+    if (err)
+      return err;
+  }
 
   if (pid != (hwloc_pid_t) -1 && pid != 0) {
     if (hwloc_topology_set_pid(topology, pid)) {
@@ -592,11 +608,9 @@ main (int argc, char *argv[])
       break;
 #endif /* CAIRO_HAS_SVG_SURFACE */
 #endif /* HWLOC_HAVE_CAIRO */
-#ifdef HWLOC_HAVE_XML
     case LSTOPO_OUTPUT_XML:
       output_xml(topology, filename, logical, legend, verbose_mode);
       break;
-#endif
     default:
       fprintf(stderr, "file format not supported\n");
       usage(callname, stderr);
